import greenfoot.*;
import java.util.List;
import java.util.ArrayList;

/**
 * This class controls the drawing of the pattern on the world background.  It creates a number of arms determined 
 * by the value of the static int field 'armCount' and lists them in an array.  It uses that array to control each
 * arm and draw the pattern.<br><br>
 * AUTHOR: danpost<br>
 * CREATED: October 6, 2013<br>
 * UPDATED: October 7, 2013<br>
 */
public class Pin extends Actor
{
    public static int armCount = 2; // the number of arms that are to be placed in the world
    List<Arm> arms = new ArrayList<Arm>(); // the list of arms created (size being the value of 'armCount')
    
    /**
     * This constructor sets its image and creates the Arm objects that are to be placed in the world,
     * listing them in the 'arms' array.  The new Arms are passed its random but restricted length
     * (do not want the sum of the lengths of all the arms to exceed half the window size), its random
     * but limited rotational speed and its not so random color (the color of the arm and the color that
     * this arm will draw).
     */
    public Pin()
    {
        // create the image of the pin
        GreenfootImage image = new GreenfootImage(7, 7);
        image.drawOval(0, 0, 6, 6);
        setImage(image);
        // create the arms and add them to the 'arms' array
        for (int i=0; i<armCount; i++)
        {
            Arm arm = new Arm(Greenfoot.getRandomNumber(145/armCount)+145/armCount,
                                 Greenfoot.getRandomNumber(6)+1,
                                 new Color((200/armCount)*((i+1)%armCount),
                                           (200/armCount)*((i+2)%armCount),
                                           (200/armCount)*(i+3)%armCount));
            arms.add(arm);
        }
    }
    
    /**
     * Called by the system when the Pin object is added to the world, this method adds the Arm objects
     * created in the constructor and added to the 'arms' array into the world at their appropriate locations.
     *
     * @param world: the World object that this Pin object was added into
     */
    public void addedToWorld(World world)
    {
        int dist = 0; // to hold the running sum of arm lengths, used for proper placement of arms into the world
        for (Arm arm : arms) // for each arm in the 'arms' list
        {
            getWorld().addObject(arm, getX()+dist, getY());
            dist += arm.length; // adds arm length to running sum of lengths
        }
    }
    
    /**
     * Turns all the arms by their rotational speed and draw the lines created by the changes in location
     * of the drawing end of the arms. Aside from rotating the arms, the arms need to be repositioned end
     * to end with each other.  This is accomplished in the 'plotPoints' method.
     */
    public void act()
    {
        if (arms.isEmpty()) return;
        for (Arm arm : arms) arm.turn(arm.speed); // rotates all arms by their respective rotational speeds
        plotPoints(); // draws the lines from last drawing end location to new one for all arms
        for (Arm arm : arms) if (arm.getRotation() != 0) return;
        getWorld().removeObjects(arms);
        arms.clear();
    }

    /**
     * Creates and adds to the world a temporary unnamed Actor object to follow the arms outward to properly
     * position arms and aid in determining where to draw the line segments of each arm.  The unnamed Actor
     * object is removed after the steps are completed.
     */
    private void plotPoints()
    {
        Actor tracer = new Actor(){}; // creates an unnamed Actor object; used for easy coordinate access
        getWorld().addObject(tracer, getX(), getY()); // adds 'tracer' into the world at Pin object location
        for (Arm arm : arms) // for each arm in the 'arms' list
        {
            arm.setLocation(tracer.getX(), tracer.getY()); // sets location of current arm at 'tracer' location
            tracer.setRotation(arm.getRotation()); // sets rotation of 'tracer' to that of the current arm
            tracer.move(arm.length); // moves 'tracer' object to drawing end of current arm
            getWorld().getBackground().setColor(arm.penColor); // set drawing color of world background
            getWorld().getBackground().drawLine(arm.lastX, arm.lastY, tracer.getX(), tracer.getY()); // draws the line segment
            arm.lastX = tracer.getX(); // have current arm save its latest x-coordinate of drawing end
            arm.lastY = tracer.getY(); // have current arm save its latest y-coordinate of drawing end
        }
        getWorld().removeObject(tracer); // remove unnamed Actor object from the world
    }
    
    /**
     * The Actor objects created from this class are used to determine the distances (constant) and angular direction
     * (changing) between the lines drawn on the background at any time.  It is also used to hold important information;
     * such as the color the arm draws and where the drawing end was last located.<br><br>
     * AUTHOR: danpost<br>
     * CREATED: October 6, 2013<br>
     * UPDATED: October 7, 2013<br>
     */
    public class Arm extends Actor
    {
        Color penColor = null; // assigned through constructor argument
        int speed, length; // assigned through constructor arguments
        int lastX, lastY; // assigned when added to world in 'addedToWorld' method
        
        /**
         * The constructor saves the argument values and creates the images for its actors
         *
         * @param armLength: the length that the arm is to be (half the image width);
         * @param rotSpeed: the absolute value of the rotational speed that the arm will move each act cycle
         * @param color: the color the image of the arm will be; as well as the color the arm will draw
         */
        public Arm(int armLength, int rotSpeed, Color color)
        {
            // save the argument values for this arm
            length = armLength;
            speed = rotSpeed*(1-2*Greenfoot.getRandomNumber(2));
            penColor = color;
            // create the image to be used for this arm
            setImage(new GreenfootImage(length*2, 3));
            getImage().setColor(penColor);
            getImage().fillRect(length, 0, length, 3);
        }
        
        /**
         * Called by the system when an Arm object is placed within a world, this method saves the initial
         * coordinates of the drawing end of the arm.
         *
         * @param world: the World object that this Arm object was added into
         */
        public void addedToWorld(World world)
        {
            lastX = getX()+getImage().getWidth()/2;
            lastY = getY();
        }
    }
}